import pygame

from math import *

def bob(pos,angle,magnitude):
    x,y = pos
    a = radians(angle)
    dy = cos(a)*magnitude
    return dy

class Tile:
    tileimages = {}
    def __init__(self,screen,pos,height,image):
        self.screen = screen
        self.pos = pos
        self.height = height
        self.gpos = ((pos[0])//40,(pos[1])//40)
        try:
            self.image = self.tileimages[image]
        except KeyError:
            ifile = "data/images/%s.png" % image
            self.image = pygame.image.load(ifile).convert_alpha()
            self.tileimages[image] = self.image
        self.rect = pygame.rect.Rect(0,0,40,height)
        self.rect.topleft = (pos[0],pos[1]+(40-height))
    def draw(self,offset):
        x,y = self.pos
        ox,oy = offset
        self.gpos = (x//40,y//40)
        self.screen.blit(self.image,(x+ox,y+oy))
    def collide(self,collision,face):
        pass

class Feather:
    def __init__(self,screen,pos):
        self.screen = screen
        self.pos = pos
        self.image = pygame.image.load("data/images/item-feather.png").convert_alpha()
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.a = 0
    def draw(self,offset):
        ox,oy = offset
        dy = bob(self.pos,self.a,2)
        self.a = (self.a+2)%360
        self.rect.center = self.pos
        x,y = self.pos
        self.screen.blit(self.image,(x+ox,y+oy+dy))
    def collide(self,collider):
        collider.feathers += 1
        return "Dead"
class GoldFeather:
    def __init__(self,screen,pos):
        self.screen = screen
        self.pos = pos
        self.image = pygame.image.load("data/images/item-gold.png").convert_alpha()
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.a = 0
    def draw(self,offset):
        ox,oy = offset
        dy = bob(self.pos,self.a,2)
        self.a = (self.a+2)%360
        self.rect.center = self.pos
        x,y = self.pos
        self.screen.blit(self.image,(x+ox,y+oy+dy))
    def collide(self,collider):
        collider.goldfeathers += 1
        return "Dead"
class Egg:
    def __init__(self,screen,pos):
        self.screen = screen
        self.pos = pos
        self.image = pygame.image.load("data/images/item-egg.png").convert_alpha()
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.a = 0
    def draw(self,offset):
        ox,oy = offset
        dy = bob(self.pos,self.a,2)
        self.a = (self.a+2)%360
        self.rect.center = self.pos
        x,y = self.pos
        self.screen.blit(self.image,(x+ox,y+oy+dy))
    def collide(self,collider):
        collider.hp = min(collider.eggs*2,collider.hp+2)
        return "Dead"
class GoldEgg:
    def __init__(self,screen,pos):
        self.screen = screen
        self.pos = pos
        self.image = pygame.image.load("data/images/item-goldegg.png").convert_alpha()
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.a = 0
    def draw(self,offset):
        ox,oy = offset
        dy = bob(self.pos,self.a,2)
        self.a = (self.a+2)%360
        self.rect.center = self.pos
        x,y = self.pos
        self.screen.blit(self.image,(x+ox,y+oy+dy))
    def collide(self,collider):
        collider.eggs += 1
        collider.hp = collider.eggs*2
        return "Dead"

class Player:
    def __init__(self,screen,pos):
        self.screen = screen
        self.pos = pos
        #Gpos is position of center
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.images = {
            "jumping":pygame.image.load("data/images/character-corky-jump.png").convert_alpha(),
            "falling":pygame.image.load("data/images/character-corky-fall.png").convert_alpha(),
            "pain":pygame.image.load("data/images/character-corky-pain.png").convert_alpha(),
            "normal":pygame.image.load("data/images/character-corky.png").convert_alpha()
            }
        self.hurtnoise = pygame.mixer.Sound("data/sound/chicken.ogg")
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox = pygame.rect.Rect(0,0,32,32)
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.2
        self.tv = 10
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0

        #Stats
        self.feathers = 0
        self.max_feathers = 0
        self.jumps = 1
        self.goldfeathers = 0
        self.hp = 2
        self.eggs = 2
    def draw(self,offset,frames):
        x,y = self.pos
        ox,oy = offset
        if (self.immunity/5)%2 == 0:
            d = True
        else:
            d = False
        if self.immunity > 0:
            i = self.images["pain"]
        elif self.state in self.images:
            i = self.images[self.state]
        else:
            i = self.images["normal"]
        if d:
            if self.facingleft:
                self.screen.blit(i,(x+ox,y+oy))
            else:
                self.screen.blit(pygame.transform.flip(i,1,0),(x+ox,y+oy))

class Shroom:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/character-shroom.png").convert_alpha()
        self.hurtnoise = pygame.mixer.Sound("data/sound/slap.ogg")
        self.target = target
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 10
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = True
        self.bounce = 1

        self.hp = 1
        self.damage = 1
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        self.screen.blit(self.image,(x+ox,y+oy))
    def ai(self,event):
        x,y = self.rect.center
        tx,ty = self.target.rect.center
        if event == "general":
            if self.target != None:
                if x > tx:
                    self.moving = -1
                elif x < tx:
                    self.moving = 1
                else:
                    self.moving = 0
class HoppinShroom:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/character-hoppin.png").convert_alpha()
        self.pimage = pygame.image.load("data/images/character-hoppin-pain.png").convert_alpha()
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/slap.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 10
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = True
        self.bounce = 1.2
        
        self.hp = 2
        self.damage = 1
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        i = self.image
        if self.immunity > 0:
            i = self.pimage
        self.screen.blit(i,(x+ox,y+oy))
    def ai(self,event):
        x,y = self.rect.center
        if event == "general":
            if self.state == "supported":
                self.dy = -8
            if self.immunity > 0:
                self.dy = 4

class MetalShroom:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/character-metal.png").convert_alpha()
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/slap.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 10
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = True
        self.bounce = 1.3

        self.hp = -1
        self.damage = 1
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        self.screen.blit(self.image,(x+ox,y+oy))
    def ai(self,event):
        x,y = self.rect.center
        if event == "general":
            if self.state == "supported":
                self.dy = -8
            if self.immunity > 0:
                self.dy = 4

class Metal2Shroom:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/character-metal.png").convert_alpha()
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/slap.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 10
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = True
        self.bounce = 1.3

        self.hp = -1
        self.damage = 1
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        self.screen.blit(self.image,(x+ox,y+oy))
    def ai(self,event):
        x,y = self.rect.center
        if event == "general":
            if self.state == "supported":
                self.dy = -14
            if self.immunity > 0:
                self.dy = 4

class Wisp:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/character-wisp-1.png").convert_alpha()
        self.image2 = pygame.image.load("data/images/character-wisp-2.png").convert_alpha()
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/hornet.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 0
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = True
        self.bounce = 1.5

        self.hp = 1
        self.damage = 1
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        i = self.image
        if (frames/5)%2 == 0:
            i = self.image2
        self.screen.blit(i,(x+ox,y+oy))
    def ai(self,event):
        pass

class Bouncepad:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/jelly-1.png").convert_alpha()
        self.pimage = pygame.image.load("data/images/jelly-2.png").convert_alpha()        
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/doing2.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 0
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.bounce = 3
        self.can_hit = True

        self.hp = -1
        self.damage = 0
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        i = self.image
        if self.immunity > 0:
            i = self.pimage
        self.screen.blit(i,(x+ox,y+oy))
    def ai(self,event):
        pass

class Bluepad:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/bjelly-1.png").convert_alpha()
        self.pimage = pygame.image.load("data/images/jelly-2.png").convert_alpha()        
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/doing.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 0
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.bounce = 2.5
        self.can_hit = True

        self.hp = -1
        self.damage = 0
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        i = self.image
        if self.immunity > 0:
            i = self.pimage
        self.screen.blit(i,(x+ox,y+oy))
    def ai(self,event):
        pass

class Orangepad:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/ojelly-1.png").convert_alpha()
        self.pimage = pygame.image.load("data/images/jelly-2.png").convert_alpha()        
        self.target = target
        self.hurtnoise = pygame.mixer.Sound("data/sound/doing.ogg")
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.05
        self.maxspeed = 1
        self.tv = 0
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.bounce = 2
        self.can_hit = True

        self.hp = -1
        self.damage = 0
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        i = self.image
        if self.immunity > 0:
            i = self.pimage
        self.screen.blit(i,(x+ox,y+oy))
    def ai(self,event):
        pass

class Lava:
    def __init__(self,screen,pos,target):
        self.screen = screen
        self.pos = pos
        self.gpos = ((pos[0]+20)//40,(pos[1]+20)//40)
        self.image = pygame.image.load("data/images/hazard-lava.png").convert_alpha()
        self.target = target
        self.hurtnoise = None
        self.hitbox = self.image.get_rect()
        self.rect = pygame.rect.Rect(0,0,40,40)
        self.rect.topleft = pos
        self.hitbox.midbottom = self.rect.midbottom
        self.dx = 0
        self.dy = 0
        self.a = 0.00
        self.maxspeed = 0
        self.tv = 0
        self.state = "falling"
        self.moving = 0
        self.facingleft = False
        self.immunity = 0
        self.can_hit = False
        self.bounce = 1

        self.hp = -1
        self.damage = 3
    def draw(self,offset,frames):
        x,y = self.hitbox.topleft
        ox,oy = offset
        self.gpos = ((self.pos[0]+20)//40,(self.pos[1]+20)//40)
        self.screen.blit(self.image,(x+ox,y+oy))
    def ai(self,event):
        pass

